<?php

namespace App\Controllers;

use App\baseClasses\KCBase;
use App\baseClasses\KCRequest;
use App\models\KCAppointment;
use App\models\KCAppointmentServiceMapping;
use App\models\KCCustomField;
use App\models\KCCustomFieldData;
use App\models\KCDoctorClinicMapping;
use App\models\KCClinicSession;
use App\models\KCClinicSchedule;
use App\models\KCPatientEncounter;
use App\models\KCServiceDoctorMapping;
use App\models\KCReceptionistClinicMapping;
use App\models\KCService;
use App\models\KCClinic;
use App\controllers\KCServiceController;

use DateTime;
use Exception;
use WP_User;

class KCDoctorController extends KCBase
{

    public $db;

    private $request;

    public function __construct()
    {

        global $wpdb;

        $this->db = $wpdb;

        $this->request = new KCRequest();

    }

    public function index()
    {

        $table_name = $this->db->prefix . 'kc_doctor_clinic_mappings';

        //check current login user permission
        if (!kcCheckPermission('doctor_list')) {
            echo json_encode([
                'status' => false,
                'status_code' => 403,
                'message' => $this->permission_message,
                'data' => []
            ]);
            wp_die();
        }

        $request_data = $this->request->getInputs();

        $args['role'] = $this->getDoctorRole();
        $args['number'] = $request_data['limit'];
        $args['offset'] = (isset($request_data['offset']) ? $request_data['offset'] : 0 );
        $args['search_columns'] = (isset($request_data['offset'])? [$request_data['searchKey']] : '');
        $args['search'] = (isset($request_data['searchValue'])? '*' . $request_data['searchValue'] . '*' : '' ) ;
        $args['orderby'] = 'ID';
        $args['order'] = 'DESC';

        if (current_user_can('administrator')) {
            $doctors = get_users($args);
        } else {
            switch ($this->getLoginUserRole()) {
                case 'kiviCare_receptionist':
                    $clinic_id = kcGetClinicIdOfReceptionist();
                    $query = "SELECT DISTINCT `doctor_id` FROM {$table_name} WHERE `clinic_id` ={$clinic_id}";
                    break;
                case 'kiviCare_clinic_admin':
                    $clinic_id = kcGetClinicIdOfClinicAdmin();
                    $query = "SELECT DISTINCT `doctor_id` FROM {$table_name} WHERE `clinic_id` ={$clinic_id}";
                    break;
            }

            $doctors = [];
            if(!empty($query)){
                // get role wise doctor id from mapping table
                $result = collect($this->db->get_results($query))->pluck('doctor_id')->toArray();
                // include mapping doctor in args array
                $args['include'] = !empty($result) ? $result : [-1];
                $doctors = get_users($args);
            }
        }

        if (!count($doctors)) {
            echo json_encode([
                'status' => false,
                'message' => esc_html__('No doctors found', 'kc-lang'),
                'data' => []
            ]);
            wp_die();
        }

        $data = [];
        foreach ($doctors as $key => $doctor) {
            $doctor->ID = (int)$doctor->ID;
            $data[$key]['ID'] = $doctor->ID;
            $image_attachment_id = get_user_meta($doctor->ID,'doctor_profile_image',true);
            $data[ $key ]['profile_image'] = (!empty($image_attachment_id) && $image_attachment_id != '') ? wp_get_attachment_url($image_attachment_id) : '';
            $data[$key]['display_name'] = $doctor->data->display_name;
            //doctor clinic name
            $clinics = $this->db->get_var("SELECT GROUP_CONCAT(clinic.name) FROM {$this->db->prefix}kc_clinics AS clinic LEFT JOIN {$this->db->prefix}kc_doctor_clinic_mappings AS dr_clinic ON dr_clinic.clinic_id = clinic.id WHERE dr_clinic.doctor_id ={$doctor->ID} GROUP BY dr_clinic.doctor_id ");
            $data[$key]['clinic_name'] = !empty($clinics) ? $clinics : "";
            $data[$key]['user_email'] = $doctor->data->user_email;
            $data[$key]['user_status'] = $doctor->data->user_status;
            $data[$key]['user_registered'] = $doctor->data->user_registered;
            $data[$key]['user_registered_formated']= date("Y-m-d", strtotime($doctor->data->user_registered));
            $user_deactivate_status = get_user_meta($doctor->ID,'kivicare_user_account_status',true);
            // verify doctor by shortcode condition
            $data[$key]['user_deactivate'] =!empty($user_deactivate_status) ? $user_deactivate_status : 'yes';
            //doctor other basic data
            $user_meta = get_user_meta($doctor->ID, 'basic_data', true);
            if(!empty($user_meta)) {
                $basic_data = json_decode($user_meta,true);
                if(!empty($basic_data)) {
                    foreach ($basic_data as $basic_data_key => $basic_data_value){
                        $data[$key][$basic_data_key] = $basic_data_key === 'specialties' ? collect($basic_data_value)->pluck('label')->implode(',') : $basic_data_value;;
                    }
                }
            }
        }

        echo json_encode([
            'status' => true,
            'message' => esc_html__('Doctors list', 'kc-lang'),
            'data' => $data,
            'total_rows' => count($doctors)
        ]);
    }

    public function save()
    {
        global $wpdb;

        //check current login user permission
        $is_permission = kcCheckPermission('doctor_add') || kcCheckPermission('doctor_profile');
        if (!$is_permission) {
            echo json_encode([
                'status' => false,
                'status_code' => 403,
                'message' => $this->permission_message,
                'data' => []
            ]);
            wp_die();
        }

        $request_data = $this->request->getInputs();
        $request_data['specialties'] = json_decode(stripslashes($request_data['specialties']));
        $request_data['qualifications'] = json_decode(stripslashes($request_data['qualifications']));
        $request_data['clinic_id'] = json_decode(stripslashes( $request_data['clinic_id']));

        $rules = [
            'first_name' => 'required',
            'last_name' => 'required',
            'user_email' => 'required|email',
            'mobile_number' => 'required',
            'gender' => 'required',
        ];

        $errors = kcValidateRequest($rules, $request_data);

        if (!empty(count($errors))) {
            echo json_encode([
                'status' => false,
                'message' => $errors[0]
            ]);
            die;
        }

        //check email condition
        $email_condition = kcCheckUserEmailAlreadyUsed($request_data);
        if(empty($email_condition['status'])){
            echo json_encode($email_condition);
            die;
        }


        $temp = [
            'mobile_number' => $request_data['mobile_number'],
            'gender' => $request_data['gender'],
            'dob' => $request_data['dob'],
            'address' => $request_data['address'],
            'city' => $request_data['city'],
            'state' => !empty($request_data['state']) ? $request_data['state'] : '',
            'country' => $request_data['country'],
            'postal_code' => $request_data['postal_code'],
            'qualifications' => !empty($request_data['qualifications']) ? $request_data['qualifications'] : [] ,
            'price_type' => $request_data['price_type'],
            'price' => $request_data['price'],
            'no_of_experience' => $request_data['no_of_experience'],
            'video_price' => isset($request_data['video_price']) ? $request_data['video_price'] : 0,
            'specialties' => !empty($request_data['specialties']) ? $request_data['specialties'] : [],
            'time_slot' => $request_data['time_slot']
        ];

        if (isset($request_data['price_type']) && $request_data['price_type'] === "range") {
            $temp['price'] = $request_data['minPrice'] . '-' . $request_data['maxPrice'];
        }

        $service_doctor_mapping = new KCServiceDoctorMapping();
        if (!isset($request_data['ID'])) {

            // create new user
            $password = kcGenerateString(12);
            $user = wp_create_user(kcGenerateUsername($request_data['first_name']), $password, sanitize_email( $request_data['user_email']) );
            $u = new WP_User($user);
            $u->display_name = $request_data['first_name'] . ' ' . $request_data['last_name'];
            wp_insert_user($u);
            //add doctor role to user
            $u->set_role($this->getDoctorRole());

            $user_id = $u->ID;

            //clinic id based on role
            $current_user_role = $this->getLoginUserRole();
            if ($current_user_role == $this->getReceptionistRole()) {
                $request_data['clinic_id'] = kcGetClinicIdOfReceptionist();
            }
            if($current_user_role == $this->getClinicAdminRole()){
                $request_data['clinic_id'] = kcGetClinicIdOfClinicAdmin();
            }

            if (isKiviCareProActive()) {
                if (gettype($request_data['clinic_id']) === 'array') {
                    foreach ($request_data['clinic_id'] as $value) {
                        $this->saveDoctorClinic($user_id,$value->id);
                    }
                }else{
                    $this->saveDoctorClinic($user_id,isset($request_data['clinic_id']) ? (int)$request_data['clinic_id'] : kcGetDefaultClinicId());
                }
            } else {
                //if pro not active default clinic id in mapping table
                $this->saveDoctorClinic($user_id,kcGetDefaultClinicId());
            }

            //get email/sms/whatsapp template dynamic key array
            $user_email_param = kcCommonNotificationUserData((int)$user_id,$password);

            //send email after doctor save
            kcSendEmail($user_email_param);

            //send sms/whatsapp after doctor save
            if(kcCheckSmsOptionEnable()){
                $sms = apply_filters('kcpro_send_sms', [
                    'type' => 'doctor_registration',
                    'user_data' => $user_email_param,
                ]);
            }

            $message = esc_html__('Doctor has been saved successfully', 'kc-lang');

            // hook for doctor save
            do_action( 'kc_doctor_save', $user_id );

        } else {

            //update doctor user
            wp_update_user(
                array(
                    'ID' => (int)$request_data['ID'],
                    'user_email' => sanitize_email( $request_data['user_email'] ),
                    'display_name' => $request_data['first_name'] . ' ' . $request_data['last_name']
                )
            );
            $request_data['ID'] = (int)$request_data['ID'];
            $user_id = $request_data['ID'];
            if (isKiviCareProActive()) {
               if(gettype($request_data['clinic_id']) === 'array'){
                   (new KCDoctorClinicMapping())->delete(['doctor_id' => $request_data['ID']]);
                   foreach ($request_data['clinic_id'] as $value) {
                       $this->saveDoctorClinic($user_id,$value->id);
                   }
               }
            }else{
                //if pro not active default clinic id in mapping table
                (new KCDoctorClinicMapping())->delete(['doctor_id' => $request_data['ID']]);
                $this->saveDoctorClinic($user_id,kcGetDefaultClinicId());
            }

            // hook for doctor update
            do_action( 'kc_doctor_update', $user_id );

            $message = __('Doctor updated successfully','kc-lang');

        }

        if ($user_id) {

            // Zoom telemed Service entry save
            if((!empty($request_data['enableTeleMed']) && (string)$request_data['enableTeleMed'] !== 'false' )  || isKiviCareGoogleMeetActive()) {

                //get telemed service id
                $telemed_Service =kcGetTelemedServiceId();

                //get doctor telemed service
                $doctor_telemed_service = $service_doctor_mapping->get_by(['service_id' => $telemed_Service, 'doctor_id' => (int)$user_id]);

                //check doctor telemed exists
                if (!empty($doctor_telemed_service) > 0) {
                    $service = new KCServiceController();
                    //get woocommerce product id of service
                    $woo_product_id = $service->getProductIdOfService($doctor_telemed_service[0]->id);
                    //check if product woocommerce product and update woo product price
                    if($woo_product_id != null &&  get_post_status( $woo_product_id )){
                        update_post_meta($woo_product_id,'_price', $request_data['video_price']);
                        update_post_meta($woo_product_id,'_sale_price', $request_data['video_price']);
                    }
                    //update doctor telemed service detail
                    $service_doctor_mapping->update(['charges' => $temp['video_price']], [
                        'service_id' => $telemed_Service,
                        'doctor_id' => (int)$user_id
                    ]);
                } else {
                    //insert doctor telemed service
                    $service_doctor_mapping->insert([
                        'service_id' => $telemed_Service,
                        'clinic_id' => kcGetDefaultClinicId(),
                        'doctor_id' => (int)$user_id,
                        'charges' => $temp['video_price']
                    ]);
                }

                //save zoom configurations details
                if(isKiviCareTelemedActive()){
                    apply_filters('kct_save_zoom_configuration', [
                        'user_id' => (int)$user_id,
                        'enableTeleMed' => $request_data['enableTeleMed'],
                        'api_key' => $request_data['api_key'],
                        'api_secret' => $request_data['api_secret']
                    ]);
                }
            }

            //update user firstname
            update_user_meta($user_id, 'first_name', $request_data['first_name']);

            //update/save user lastname
            update_user_meta($user_id, 'last_name', $request_data['last_name']);

            //update/save user other details
            update_user_meta($user_id, 'basic_data', json_encode($temp, JSON_UNESCAPED_UNICODE));

            //update/save user description
            if(isset($request_data['description'])){
                update_user_meta($user_id, 'doctor_description',$request_data['description'] );
            }

            //update/save user custom field
            if (!empty($request_data['custom_fields'])) {
                $request_data['custom_fields'] = json_decode(stripslashes( $request_data['custom_fields']));
                kvSaveCustomFields('doctor_module', $user_id, $request_data['custom_fields']);
            }

            //update/save user digital signature
            if(!empty($request_data['signature'])){
                update_user_meta($user_id ,'doctor_signature',$request_data['signature']);
            }

            //update/save doctor profile image
            if($request_data['profile_image'] != '' && isset($request_data['profile_image']) && $request_data['profile_image'] != null ) {
                $attachment_id = media_handle_upload('profile_image', 0);
                if(!is_wp_error($attachment_id)) {
                    update_user_meta( $user_id, 'doctor_profile_image',  $attachment_id  );
                } else {
                    if(gettype($request_data['profile_image']) != 'string') {
                        echo json_encode([
                            'status' => false,
                            'message' => esc_html__('Failed to upload profile image.', 'kc-lang')
                        ]); die;
                    }
                }
            }

            //update/save user status
            $wpdb->update($wpdb->base_prefix . 'users', ['user_status' => $request_data['user_status']], ['ID' => (int)$user_id]);
        }


        if (!empty($user->errors)) {
            echo json_encode([
                'status' => false,
                'message' => $user->get_error_message() ? $user->get_error_message() : esc_html__('Failed to save Doctor data.', 'kc-lang')
            ]);
        } else {
            echo json_encode([
                'status' => true,
                'message' => $message
            ]);
        }
    }

    public function edit()
    {
        $is_permission = false;

        //check current login user permission
        if (kcCheckPermission('doctor_edit') || kcCheckPermission('doctor_view') || kcCheckPermission('doctor_profile')) {
            $is_permission = true;
        }
        if(!$is_permission){
            echo json_encode([
                'status' => false,
                'status_code' => 403,
                'message' => $this->permission_message,
                'data' => []
            ]);
            wp_die();
        }

        $request_data = $this->request->getInputs();

        
        try {

            if (!isset($request_data['id'])) {
                throw new Exception(esc_html__('Data not found', 'kc-lang'), 400);
            }


            $id = (int)$request_data['id'];
            $image_attachment_id = get_user_meta($id,'doctor_profile_image',true);
            $user_image_url = wp_get_attachment_url($image_attachment_id);

            $user = get_userdata($id);
            unset($user->user_pass);

            $user_data = get_user_meta($id, 'basic_data', true);

            //remove null value from object
            if(!empty($user_data)){
                $user_data = array_map(function ($v){
                    if(is_null($v) || $v == 'null'){
                        $v = '';
                    }
                    return $v;
                },(array)json_decode($user_data));
            }else{
                $user_data = [];
            }

            $first_name = get_user_meta($id, 'first_name', true);
            $last_name = get_user_meta($id, 'last_name', true);
            $description = get_user_meta($id, 'doctor_description', true);

            $data = (object)array_merge((array)$user->data, $user_data);
            $data->first_name = $first_name;
            $data->username = $data->user_login;
            $data->description = !empty($description) ? $description : '';
            $data->last_name = $last_name;
            $data->qualifications = !empty($data->qualifications) ? $data->qualifications : [];
            $data->specialties = !empty($data->specialties) ? $data->specialties : [];
            $telemed_charges = getDoctorTelemedServiceCharges($id);
            $data->video_price = $telemed_charges;
            //doctor clinic
            $clinics = collect($this->db->get_results("SELECT clinic.id AS id ,clinic.name AS label FROM {$this->db->prefix}kc_clinics AS clinic 
            LEFT JOIN {$this->db->prefix}kc_doctor_clinic_mappings AS doctor_clinic ON doctor_clinic.clinic_id =clinic.id 
            WHERE doctor_clinic.doctor_id={$id}"))
                ->map(function ($v){
                    return ['id' => $v->id ,'label' => $v->label];
                })->toArray();
            $data->clinic_id = $clinics;
            if (isset($data->price_type)) {
                if ($data->price_type === "range") {
                    $price = explode("-", $data->price);
                    $data->minPrice = isset($price[0]) ? $price[0] : 0;
                    $data->maxPrice = isset($price[1]) ? $price[1] : 0;
                    $data->price = 0;
                }
            } else {
                $data->price_type = "range";
            }

            //doctor telemed descriptions
            if(isKiviCareTelemedActive()) {

                $config_data = apply_filters('kct_get_zoom_configuration', [
                    'user_id' => $id,
                ]);

                if (isset($config_data['status']) && $config_data['status']) {
                    $data->enableTeleMed =$config_data['data']->enableTeleMed;
                    $data->api_key = !empty($config_data['data']->api_key) && $config_data['data']->api_key !== 'null' ? $config_data['data']->api_key : '';
                    $data->api_secret = !empty($config_data['data']->api_secret) && $config_data['data']->api_secret !== 'null' ? $config_data['data']->api_secret : '';
                    $data->zoom_id = $config_data['data']->zoom_id;
                }
            }

            //doctor digital details
            $data->signature = '';
            $doctor_signature = get_user_meta($id ,'doctor_signature',true);
            if(!empty($doctor_signature)){
                $data->signature = strval($doctor_signature);
            }

            //doctor custom field
            $custom_filed = kcGetCustomFields('doctor_module', $id);
            $data->user_profile =$user_image_url;
            if ($data) {

                //check if request doctor detail is from valid user
                $current_user_role = $this->getLoginUserRole();
                $validate = true;
                if ($current_user_role == $this->getClinicAdminRole()) {
                    $clinic_id = kcGetClinicIdOfClinicAdmin();
                    $clinic_id_array =collect($clinics)->pluck('id')->map(function ($v){
                        return (int)$v;
                    })->toArray();
                    if (!in_array($clinic_id,$clinic_id_array)) {
                        $validate = false;
                    }
                } elseif ($current_user_role == $this->getReceptionistRole()) {
                    $clinic_id = kcGetClinicIdOfReceptionist();
                    $clinic_id_array =collect($clinics)->pluck('id')->map(function ($v){
                        return (int)$v;
                    })->toArray();
                    if (!in_array($clinic_id,$clinic_id_array)) {
                        $validate = false;
                    }
                } elseif ($current_user_role == $this->getDoctorRole()) {
                    if (get_current_user_id() !== $id) {
                        $validate = false;
                    }
                }

                //request doctor detail is not from valid user redirect to 403 page
                if (!$validate) {
                    echo json_encode( [
                        'status' => false,
                        'status_code' => 403,
                        'message' => esc_html__('You don\'t have permission to access', 'kc-lang'),
                        'data' => []
                    ]);
                    die;
                }
                echo json_encode([
                    'status' => true,
                    'message' => 'Doctor data',
                    'id' => $id,
                    'user_data' => $user_data,
                    'data' => $data,
                    'custom_filed'=>$custom_filed
                ]);

            } else {
                throw new Exception(esc_html__('Data not found', 'kc-lang'), 400);
            }


        } catch (Exception $e) {

            $code = $e->getCode();
            $message = $e->getMessage();

            header("Status: $code $message");

            echo json_encode([
                'status' => false,
                'message' => $message
            ]);
        }
    }

    public function delete()
    {

        //check current login user permission
        if (!kcCheckPermission('doctor_delete')) {
            echo json_encode([
                'status' => false,
                'status_code' => 403,
                'message' => $this->permission_message,
                'data' => []
            ]);
            wp_die();
        }

        $request_data = $this->request->getInputs();

        try {

            if (!isset($request_data['id'])) {
                throw new Exception(esc_html__('Data not found', 'kc-lang'), 400);
            }

            $id = (int)$request_data['id'];

            //delete doctor zoom telemed service
            if (isKiviCareTelemedActive()) {
                apply_filters('kct_delete_patient_meeting', ['doctor_id' => $id]);
            }

            // hook for doctor delete
            do_action( 'kc_doctor_delete', $id );

            //delete all doctor related entry

            //delete doctor holiday
            (new KCClinicSchedule())->delete(['module_id' => $id, 'module_type' => 'doctor']);

            //delete doctor clinic session
            (new KCClinicSession())->delete(['doctor_id' => $id]);

            //doctor clinic mapping entry
            (new KCDoctorClinicMapping())->delete(['doctor_id' => $id]);

            // send appointment cancel notification  based on conditiom
            $allAppointments = $this->db->get_results("SELECT * FROM {$this->db->prefix}kc_appointments WHERE doctor_id={$id}");
            if(!empty($allAppointments)){
                foreach ($allAppointments as $res){
                    //check if appointment date and time greater then current time and status is booked
                    if($res->appointment_start_date >= current_time('Y-m-d') && in_array($res->status,['1',1])){
                        $email_data = kcCommonNotificationData($res,[],'','cancel_appointment');
                        //send cancel email
                        kcSendEmail($email_data);
                        if(kcCheckSmsOptionEnable() || kcCheckWhatsappOptionEnable()){
                            apply_filters('kcpro_send_sms', [
                                'type' => 'cancel_appointment',
                                'appointment_id' => $res->id,
                            ]);
                        }
                    }
                    //cancel zoom meeting
                    if(isKiviCareTelemedActive()){
                        apply_filters('kct_delete_appointment_meeting', ['id'=>$res->id]);
                    }
                    //remove google calendar event
                    if(kcCheckGoogleCalendarEnable()){
                        apply_filters('kcpro_remove_appointment_event', ['appoinment_id'=>$res->id]);
                    }

                    //remove google meet event
                    if(isKiviCareGoogleMeetActive()){
                        apply_filters('kcgm_remove_appointment_event',['appoinment_id' => $res->id]);
                    }
                    //delete appointment service mappings
                    (new KCAppointmentServiceMapping())->delete(['appointment_id' => $res->id]);

                    //delete appointment custom fields
                    (new KCCustomFieldData())->delete(['module_type' => 'appointment_module','module_id' => $res->id]);
                }
            }
            //delete doctor appointment
            (new KCAppointment())->delete(['doctor_id' => $id]);
            //delete doctor encounter
            (new KCPatientEncounter())->delete(['doctor_id' => $id]);

            // delete woocommerce product on service delete
            collect((new KCServiceDoctorMapping())->get_by(['doctor_id' => $id]))->pluck('id')->map(function($v){
                $product_id = kivicareGetProductIdOfService($v);
                if($product_id != null && get_post_status( $product_id )){
                    do_action( 'kc_woocoomerce_service_delete', $product_id );
                    wp_delete_post($product_id);
                }
                return $v;
            });
            //delete doctor service
            (new KCServiceDoctorMapping())->delete(['doctor_id' => $id]);
            //delete current doctor custom field
            (new KCCustomField())->delete(['module_type' => 'appointment_module','module_id' => $id]);
            (new KCCustomFieldData())->delete(['module_type' => 'doctor_module','module_id' => $id]);

            //delete doctor usermeta
            delete_user_meta($id, 'basic_data');
            delete_user_meta($id, 'first_name');
            delete_user_meta($id, 'last_name');
            //delete user
            $results = wp_delete_user($id);
            if ($results) {
                echo json_encode([
                    'status' => true,
                    'message' => esc_html__('Doctor has been deleted successfully', 'kc-lang'),
                ]);
            } else {
                throw new Exception(esc_html__('Data not found', 'kc-lang'), 400);
            }


        } catch (Exception $e) {

            $code = $e->getCode();
            $message = $e->getMessage();

            header("Status: $code $message");

            echo json_encode([
                'status' => false,
                'message' => $message
            ]);
        }
    }

    public function getDoctorWorkdays(){
        $request_data = $this->request->getInputs();
        $results = [];
        $status = false;
        $login_user_role = $this->getLoginUserRole();

        if($this->getDoctorRole() === $login_user_role) {
            $request_data['doctor_id'] = get_current_user_id() ;
        }

        if(isKiviCareProActive()){
            if($login_user_role == 'kiviCare_clinic_admin'){
                $request_data['clinic_id'] = kcGetClinicIdOfClinicAdmin();
            }elseif ($login_user_role == 'kiviCare_receptionist') {
                $request_data['clinic_id'] = kcGetClinicIdOfReceptionist();
            }
        }


        //check doctor and clini id exists in request data
        if(isset($request_data['clinic_id']) && $request_data['clinic_id'] != '' &&
            isset($request_data['doctor_id']) && $request_data['doctor_id'] != ''){

            $request_data['doctor_id'] = (int)$request_data['doctor_id'];
            $request_data['clinic_id'] = (int)$request_data['clinic_id'];


            //get doctor session days
            $results = collect($this->db->get_results("SELECT DISTINCT day FROM {$this->db->prefix}kc_clinic_sessions where doctor_id={$request_data['doctor_id']} AND clinic_id={$request_data['clinic_id']}"))->pluck('day')->toArray();

            //week day for php vue appointment dashboard
            $days = [1 => 'sun', 2 => 'mon', 3 =>'tue', 4 => 'wed', 5 => 'thu', 6 => 'fri', 7 => 'sat'];
            //week day for php shortcode widget
            if(!empty($request_data['type']) && $request_data['type'] === 'flatpicker'){
                $days = [0 => 'sun', 1 => 'mon', 2 =>'tue', 3 => 'wed', 4 => 'thu', 5 => 'fri', 6 => 'sat'];
            }


            if(count($results) > 0){
                // get unavilable  days
               $results = array_diff(array_values($days),$results);
               //get key of unavilable days
               $results = array_map(function ($v) use ($days){
                   return array_search($v,$days);
               },$results);
               $results = array_values($results);
            }
            else{
                //get all days keys
                $results = array_keys($days);
            }
            $status = true;
        }
        echo json_encode([
            'status' => $status,
             'data' => $results
        ]);
    }

    public function getDoctorWorkdayAndSession(){
        $request_data = $this->request->getInputs();
        $doctors_sessions = doctorWeeklyAvailability(['clinic_id'=>$request_data['clinic_id'],'doctor_id'=>$request_data['doctor_id']]);
        echo json_encode([
            'data' => $doctors_sessions,
            'status' => true
        ]);
    }

    public function saveDoctorClinic($doctor_id,$clinic_id){
        //save/update doctor clinic mappings
        $doctor_mapping = new KCDoctorClinicMapping;
        $new_temp = [
            'doctor_id' => (int)$doctor_id,
            'clinic_id' => (int)$clinic_id,
            'owner' => 0,
            'created_at' => current_time('Y-m-d H:i:s')
        ];
        $doctor_mapping->insert($new_temp);
    }
}
